#include <LiquidCrystal.h>

const int btn = 2;  //button input pin

const int sAudioPin = 7;

//texts to display
char* introText[] = {"Jump over the", "PEOPLE!!!", "Hit the button", "To start game!"};

//variable declarations
int game = 0;
int buttonPress = 0; 
int buttonState = 0;
int obstaclePos;
int obsDelay;
int score = 0;
int maxScore = 0;
int nrPlays = 0;
int lvl = 0;
int introMsg = 1;

//GRAPHICS
byte runnin1[8] = {
  0b01100,
  0b01100,
  0b00000,
  0b11100,
  0b01110,
  0b01100,
  0b11110,
  0b00011
};
byte runnin2[8] = {
  0b00110,
  0b00110,
  0b00000,
  0b00111,
  0b01110,
  0b00110,
  0b01110,
  0b00101
};
byte jump[8] = {
  0b00100,
  0b01100,
  0b11100,
  0b01110,
  0b01100,
  0b11111,
  0b00000,
  0b00001
};
byte def1[8] = {
  0b00100,
  0b00110,
  0b00111,
  0b01110,
  0b00110,
  0b01111,
  0b01000,
  0b00000
};
byte def2[8] = {
  0b00100,
  0b00110,
  0b00111,
  0b01110,
  0b00110,
  0b00110,
  0b00110,
  0b00010
};
byte slide[8] = {
  0b00000,
  0b00000,
  0b00011,
  0b00011,
  0b00000,
  0b01111,
  0b11111,
  0b00000
};

 
LiquidCrystal lcd(4, 6, 10, 11, 12, 13);
void setup() {

  Serial.begin(9600);
  pinMode(btn,INPUT_PULLUP);
  lcd.begin(16, 2);
  lcd.createChar(0, runnin1);
  lcd.createChar(1, runnin2);
  lcd.createChar(2, jump);
  lcd.createChar(3, def1);
  lcd.createChar(4, def2);
  lcd.createChar(5, slide);
}

void loop()
{
    if (introMsg) {
      intro();
      introMsg = 0;
    } else {
      nextTry();
    }
    
    if (game == 1) 
     gameOn();
  
    endGame(); 
}

 void intro()
 {
        score = 0;
        lcd.clear();
        lcd.print(introText[0]);
        lcd.setCursor(1, 1);
       lcd.print (introText[1]);
      delay(2000);
   
      lcd.clear();
        lcd.print("Made by Gabriel");
      lcd.setCursor(1, 1);
      lcd.print ("Coculescu");
        delay(2000);
        lcd.clear();
   
        while (button() != 1)            //wait for the user to hit the button, display the text until that.
        {
         lcd.setCursor(0, 0);
         lcd.print(introText[2]);
         lcd.setCursor(1, 1);
       lcd.print (introText[3]);
         delay(100);
        }
   
        if (button() == 1)
          game = 1;
 }

 void nextTry()
 {
      score = 0;
      nrPlays++;
      lcd.clear();
        lcd.print(String("Nr of plays: ") + String(nrPlays));
        lcd.setCursor(1,1);
      lcd.print (String("Max Score: ") + String(maxScore));
      delay(2500);
      lcd.clear();
   
        while (button() != 1)            //wait for the user to hit the button, display the text until that.
        {
         lcd.setCursor(0, 0);
         lcd.print(introText[2]);
         lcd.setCursor(1, 1);
       lcd.print (introText[3]);
         delay(100);
        }
   
        if (button() == 1)
          game = 1;
 }

 void reset()
 {
      if(obstaclePos % 2 == 0)            //Changing the display of the player from one pose to another to show
                      //  he is running, smartly using obstaclePos.
      {
          lcd.clear();
          lcd.setCursor(0, 1);
          lcd.write(byte(0));
      }
      else
      {
          lcd.clear();
          lcd.setCursor(0, 1);
          lcd.write(1);
      }
 }

 void gameOn()
 {
  obsDelay = 260;   //initial delay for the defender
  obstaclePos = 15;  //since it is a 16x2 lcd, and index starts from 0.
  while (game == 1)
  {
       reset();
       if (button() == 1)      //if button is pressed.
        { //
          tone(sAudioPin, 600);
          delay(100);
          noTone(sAudioPin);
         
         //
          int jumpDst = 3;
          lcd.setCursor(0, 1);
          while (jumpDst > 0) {  //print the player jumping
            lcd.clear();
            lcd.setCursor(0,0);
            lcd.write(2);
            obstacle(obstaclePos); //print the defender 
            obstaclePos--;
            delay(obsDelay);
            jumpDst--;
          }
     
          lcd.clear();
          lcd.setCursor(0, 1);
          lcd.write(1);            //print the player back on ground
          obstacle(obstaclePos);   //print the defender  
          
        }
        else
        {
          if(obstaclePos != 0)        //if the defender has not approached the player, then continue.
          {
              reset();
              obstacle(obstaclePos);
          }
          else if(obstaclePos == 0)   //if the defender is at 0 position,he has approached the player
                        //when he is on ground, then end the game.
          {
             game = 0;
             break;
          }
        }
        obstaclePos--;               //brings the defender closer to the player
        if(obstaclePos < 0)            //if the player successfully dodged then,  
        {                             
          obsDelay -= 20;       //decrease the delay (increase speed),
          obstaclePos = 15;             //bring the defender to starting position,
          score++;                    //and increase the score by 1.
        }
        if(obsDelay == 0)               //if 13 players have been dodged (because 260/20=13), then end the game
            game = 0;
             
        delay(obsDelay);
    }
  }
int button() 
{
       buttonState = digitalRead(btn);
  
       Serial.println(buttonState);
  
       if (buttonState == HIGH) {
           return 0;
         } else return 1;
}
void obstacle(int i)
{ 
          if(obstaclePos % 3 == 0&& obstaclePos > 3)   //if defender is far away from the player, then show him running
          {
              lcd.setCursor(i, 1);
              lcd.write(3);
          }
          else if (obstaclePos % 3 != 0 && obstaclePos > 3)
          {
              lcd.setCursor(i, 1);
              lcd.write(4);
          }
          else                                 //if he is 4 steps close to the player, then show him sliding
          {
              lcd.setCursor(i, 1);
              lcd.write(5);
          }
        lcd.setCursor(7, 0);
        lcd.print("Score: ");
          lcd.print(score);
}
void endGame()
{
  if (score > maxScore) {
    
      maxScore = score;
    
      lcd.clear();
      tone(sAudioPin, 770);
      lcd.setCursor(0, 0);
      lcd.print("CONGRATULATIONS!");
      delay(850);
      lcd.clear();
      tone(sAudioPin, 660);
      lcd.setCursor(0, 0);
      lcd.print("YOU'VE JUST SET ");
      delay(780);
      lcd.clear();
      tone(sAudioPin, 500);
      lcd.setCursor(0, 0);
      lcd.print("A NEW MAX_SCORE: ");
      delay(680);
      lcd.clear();
      tone(sAudioPin, 350);
      lcd.setCursor(7, 0);
      lcd.print(maxScore);
      delay(600);
      lcd.clear();
      noTone(sAudioPin);
      
  } else {
      
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("BETTER LUCK!");
      lcd.setCursor(1, 2);
      lcd.print("NEXT GAME!!!");
      tone(sAudioPin,550);
      delay(250);
      tone(sAudioPin,400);
      delay(350);
      tone(sAudioPin,350);
      delay(500);
      lcd.clear();
      noTone(sAudioPin);      

  }
  
    if(score == 13)
    {
        lcd.clear();
        lcd.print("Congratulations!");
        lcd.setCursor(0, 1);
        lcd.print("You won!");
        delay(3000);
    }
    else if(score < 4)
    {
        lcd.clear();
        lcd.print("You can more!");
        lcd.setCursor(0, 1);
        lcd.print("Score: ");
        lcd.print(score);
        delay(3000);
    }
    else if(score < 8)
    {
        lcd.clear();
        lcd.print("Not bad!");
        lcd.setCursor(0,1);
        lcd.print("Score: ");
        lcd.print(score);
        delay(3000);
    }
    else
    {
        lcd.clear();
        lcd.print("Great!");
        lcd.setCursor(0, 1);
        lcd.print("Score: ");
        lcd.print(score);
        delay(3000);
    }
}
